package eu.eudat.dao.databaselayer.context;

import eu.eudat.entities.DataEntity;
import eu.eudat.queryable.QueryableList;
import eu.eudat.queryable.hibernatequeryablelist.QueryableHibernateList;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.*;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import java.util.List;
import java.util.UUID;


@Repository("databaseCtx")
public class DatabaseContext<T extends DataEntity<T>> {

    @PersistenceContext
    private EntityManager entityManager;

    @Autowired
    public DatabaseContext(EntityManagerFactory entityManagerFactory) {
        this.entityManager = entityManagerFactory.createEntityManager();
    }

    public QueryableList<T> getQueryable(Class<T> type) {
        return new QueryableHibernateList<>(this.entityManager, type).setEntity(type);
    }

    @Transactional
    public T createOrUpdate(T item, Class<T> type) {
        EntityManager entityManager = this.entityManager;
        if (item.getKeys()[0] != null) {
            T oldItem = entityManager.find(type, item.getKeys()[0]);
            if(oldItem!=null) {
                oldItem.update(item);
                entityManager.merge(oldItem);
                return oldItem;
            }else {
                entityManager.persist(item);
            }
        } else entityManager.persist(item);
        return item;
    }

    public long count(Class<T> entityClass) {
        EntityManager entityManager = this.entityManager;
        return ((Number) entityManager.createQuery("select count(e) from " + entityClass.getSimpleName() + " e").getSingleResult()).longValue();
    }

    public void delete(T item){
        this.entityManager.remove(item);
    }
}
