package eu.eudat.dao.entities;

import java.util.UUID;

import eu.eudat.dao.DatabaseAccess;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.models.criteria.DataRepositoryCriteria;
import eu.eudat.queryable.QueryableList;

import eu.eudat.entities.DataRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("dataRepositoryDao")
public class DataRepositoryDaoImpl extends DatabaseAccess<DataRepository> implements DataRepositoryDao {

    @Autowired
    public DataRepositoryDaoImpl(DatabaseService<DataRepository> databaseService) {
        this.setDatabaseService(databaseService);
    }

    @Override
    public QueryableList<DataRepository> getWithCriteria(DataRepositoryCriteria criteria) {
        QueryableList<DataRepository> query = this.getDatabaseService().getQueryable(DataRepository.class);
        if (criteria.getLike() != null)
            query.where((builder, root) -> builder.equal(root.get("reference"), criteria.getLike()));
        return query;
    }

    @Override
    public DataRepository find(UUID id) {
        return this.getDatabaseService().getQueryable(DataRepository.class).where((builder, root) -> builder.equal(root.get("id"), id)).getSingle();
    }

    @Override
    public DataRepository createOrUpdate(DataRepository item) {
        return getDatabaseService().createOrUpdate(item, DataRepository.class);
    }

    @Override
    public void delete(DataRepository item) {
        this.getDatabaseService().delete(item);
    }

    @Override
    public QueryableList<DataRepository> asQueryable() {
        return this.getDatabaseService().getQueryable(DataRepository.class);
    }
}
