package eu.eudat.dao.entities;
import eu.eudat.dao.DatabaseAccess;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.entities.DatasetProfile;
import eu.eudat.entities.ExternalDataset;
import eu.eudat.entities.Project;
import eu.eudat.models.criteria.ExternalDatasetCriteria;
import eu.eudat.queryable.QueryableList;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.UUID;


@Component("externalDatasetDao")
public class ExternalDatasetDaoImpl extends DatabaseAccess<ExternalDataset> implements ExternalDatasetDao {

    @Autowired
    public ExternalDatasetDaoImpl(DatabaseService<ExternalDataset> databaseService) {
        this.setDatabaseService(databaseService);
    }

    @Override
    public QueryableList<ExternalDataset> getWithCriteria(ExternalDatasetCriteria criteria) {
        QueryableList<ExternalDataset> query = this.getDatabaseService().getQueryable(ExternalDataset.class);
        if (criteria.getLike() != null && !criteria.getLike().isEmpty()) query.where((builder, root) -> builder.like(root.get("label"), "%" + criteria.getLike() + "%"));
        return query;
    }

    @Override
    public ExternalDataset createOrUpdate(ExternalDataset item) {
        return this.getDatabaseService().createOrUpdate(item,ExternalDataset.class);
    }

    @Override
    public ExternalDataset find(UUID id) {
        return this.getDatabaseService().getQueryable(ExternalDataset.class).where((builder, root) -> builder.equal(root.get("id"),id)).getSingle();
    }

    @Override
    public void delete(ExternalDataset item) {
        this.getDatabaseService().delete(item);
    }

    @Override
    public QueryableList<ExternalDataset> asQueryable() {
        return this.getDatabaseService().getQueryable(ExternalDataset.class);
    }
}
