package eu.eudat.dao.entities;

import java.util.UUID;

import eu.eudat.dao.DatabaseAccess;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.entities.Invitation;
import eu.eudat.models.criteria.OrganisationCriteria;
import eu.eudat.queryable.QueryableList;

import eu.eudat.entities.Organisation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("organisationDao")
public class OrganisationDaoImpl extends DatabaseAccess<Organisation> implements OrganisationDao {

    @Autowired
    public OrganisationDaoImpl(DatabaseService<Organisation> databaseService) {
        this.setDatabaseService(databaseService);
    }

    @Override
    public QueryableList<Organisation> getWithCriteria(OrganisationCriteria criteria) {
        QueryableList<Organisation> query = this.getDatabaseService().getQueryable(Organisation.class);
        if (criteria.getLike() != null)
            query.where((builder, root) -> builder.equal(root.get("reference"), criteria.getLike()));
        return query;
    }

    @Override
    public Organisation createOrUpdate(Organisation item) {
        return this.getDatabaseService().createOrUpdate(item,Organisation.class);
    }

    @Override
    public Organisation find(UUID id) {
        return this.getDatabaseService().getQueryable(Organisation.class).where((builder, root) -> builder.equal(root.get("id"), id)).getSingle();
    }

    @Override
    public void delete(Organisation item) {
        this.getDatabaseService().delete(item);
    }

    @Override
    public QueryableList<Organisation> asQueryable() {
        return this.getDatabaseService().getQueryable(Organisation.class);
    }
}
