package eu.eudat.dao.entities;

import java.util.UUID;

import eu.eudat.dao.DatabaseAccess;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.entities.*;
import eu.eudat.models.criteria.ProjectCriteria;
import eu.eudat.queryable.QueryableList;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("projectDao")
public class ProjectDaoImpl extends DatabaseAccess<Project> implements ProjectDao {

    @Autowired
    public ProjectDaoImpl(DatabaseService<Project> databaseService) {
        this.setDatabaseService(databaseService);
    }

    @Override
    public QueryableList<Project> getWithCriteria(ProjectCriteria criteria) {
        QueryableList<Project> query = getDatabaseService().getQueryable(Project.class);
        if (criteria.getLike() != null && !criteria.getLike().isEmpty())
            query.where((builder, root) -> builder.like(root.get("label"), "%" + criteria.getLike() + "%"));
        if (criteria.getPeriodEnd() != null)
            query.where((builder, root) -> builder.lessThan(root.get("enddate"), criteria.getPeriodEnd()));
        if (criteria.getPeriodStart() != null)
            query.where((builder, root) -> builder.greaterThan(root.get("startdate"), criteria.getPeriodStart()));
        if (criteria.getReference() != null)
            query.where((builder, root) -> builder.equal(root.get("reference"), criteria.getReference()));
        query.where((builder, root) -> builder.notEqual(root.get("status"), Project.Status.DELETED.getValue()));
        return query;
    }

    @Override
    public Project createOrUpdate(Project item) {
        return getDatabaseService().createOrUpdate(item, Project.class);
    }

    @Override
    public Project find(UUID id) {
        return getDatabaseService().getQueryable(Project.class).where((builder, root) -> builder.equal((root.get("id")), id)).getSingle();
    }

    @Override
    public Long count() {
        return this.getDatabaseService().count(Project.class);
    }

    @Override
    public void delete(Project item) {
        this.getDatabaseService().delete(item);
    }

    @Override
    public QueryableList<Project> asQueryable() {
        return this.getDatabaseService().getQueryable(Project.class);
    }

    public QueryableList<Project> getAuthenticated(QueryableList<Project> query, UserInfo principal) {
        query.where((builder, root) -> builder.equal(root.get("creator"), principal));
        return query;
    }
}
