package eu.eudat.dao.entities;

import java.util.UUID;

import eu.eudat.dao.DatabaseAccess;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.entities.Organisation;
import eu.eudat.models.criteria.RegistryCriteria;
import eu.eudat.queryable.QueryableList;

import eu.eudat.entities.Registry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("registryDao")
public class RegistryDaoImpl extends DatabaseAccess<Registry> implements RegistryDao {

	@Autowired
	public RegistryDaoImpl(DatabaseService<Registry> databaseService) {
		this.setDatabaseService(databaseService);
	}

	@Override
	public QueryableList<Registry> getWithCriteria(RegistryCriteria criteria) {
		QueryableList<Registry> query = this.getDatabaseService().getQueryable(Registry.class);
		if(criteria.getLike()!=null) query.where((builder, root) -> builder.equal(root.get("reference"),criteria.getLike()));
		return query;
	}

	@Override
	public Registry createOrUpdate(Registry item) {
		return this.getDatabaseService().createOrUpdate(item,Registry.class);
	}

	@Override
	public Registry find(UUID id) {
		return this.getDatabaseService().getQueryable(Registry.class).where((builder, root) -> builder.equal(root.get("id"),id)).getSingle();
	}

	@Override
	public void delete(Registry item) {
		this.getDatabaseService().delete(item);
	}

	@Override
	public QueryableList<Registry> asQueryable() {
		return this.getDatabaseService().getQueryable(Registry.class);
	}
}
