package eu.eudat.dao.entities;

import java.util.UUID;

import eu.eudat.dao.DatabaseAccess;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.entities.Organisation;
import eu.eudat.models.criteria.ResearcherCriteria;
import eu.eudat.queryable.QueryableList;

import eu.eudat.entities.Researcher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("researcherDao")
public class ResearcherDaoImpl extends DatabaseAccess<Researcher> implements ResearcherDao {

	@Autowired
	public ResearcherDaoImpl(DatabaseService<Researcher> databaseService) {
		this.setDatabaseService(databaseService);
	}

	@Override
	public QueryableList<Researcher> getWithCriteria(ResearcherCriteria criteria) {
		QueryableList<Researcher> query = this.getDatabaseService().getQueryable(Researcher.class);
		if(criteria.getLike()!=null) query.where((builder, root) -> builder.equal(root.get("reference"),criteria.getLike()));
		return query;
	}

	@Override
	public Researcher createOrUpdate(Researcher item) {
		return this.getDatabaseService().createOrUpdate(item,Researcher.class);
	}

	@Override
	public Researcher find(UUID id) {
		return this.getDatabaseService().getQueryable(Researcher.class).where((builder, root) -> builder.equal(root.get("id"),id)).getSingle();
	}

	@Override
	public void delete(Researcher item) {
		this.getDatabaseService().delete(item);
	}

	@Override
	public QueryableList<Researcher> asQueryable() {
		return this.getDatabaseService().getQueryable(Researcher.class);
	}
}
