package eu.eudat.dao.entities;

import java.util.UUID;

import eu.eudat.dao.DatabaseAccess;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.entities.Organisation;
import eu.eudat.models.criteria.ServiceCriteria;
import eu.eudat.queryable.QueryableList;

import eu.eudat.entities.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("serviceDao")
public class ServiceDaoImpl extends DatabaseAccess<Service> implements ServiceDao {

	@Autowired
	public ServiceDaoImpl(DatabaseService<Service> databaseService) {
		this.setDatabaseService(databaseService);
	}

	@Override
	public QueryableList<Service> getWithCriteria(ServiceCriteria criteria) {
		QueryableList<Service> query = this.getDatabaseService().getQueryable(Service.class);
		if(criteria.getLike()!=null) query.where((builder, root) -> builder.equal(root.get("reference"),criteria.getLike()));
		return query;
	}

	@Override
	public Service createOrUpdate(Service item) {
		return this.getDatabaseService().createOrUpdate(item,Service.class);
	}

	@Override
	public Service find(UUID id) {
		return this.getDatabaseService().getQueryable(Service.class).where((builder, root) -> builder.equal(root.get("id"),id)).getSingle();
	}

	@Override
	public void delete(Service item) {
		this.getDatabaseService().delete(item);
	}

	@Override
	public QueryableList<Service> asQueryable() {
		return this.getDatabaseService().getQueryable(Service.class);
	}
}
