package eu.eudat.dao.entities;

import java.util.Arrays;
import java.util.UUID;

import eu.eudat.dao.DatabaseAccess;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.entities.Organisation;
import eu.eudat.entities.UserInfo;
import eu.eudat.models.criteria.UserInfoCriteria;
import eu.eudat.queryable.QueryableList;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("userInfoDao")
public class UserInfoDaoImpl extends DatabaseAccess<UserInfo> implements UserInfoDao {

    @Autowired
    public UserInfoDaoImpl(DatabaseService<UserInfo> databaseService) {
        this.setDatabaseService(databaseService);
    }

    @Override
    public QueryableList<UserInfo> getWithCriteria(UserInfoCriteria criteria) {
        QueryableList<UserInfo> users = this.getDatabaseService().getQueryable(UserInfo.class);
        if (criteria.getAppRoles() != null && !criteria.getAppRoles().isEmpty())
            users.where((builder, root) -> root.join("userRoles").get("role").in(criteria.getAppRoles()));
        if (criteria.getLike() != null)
            users.where((builder, root) -> builder.or(builder.like(root.get("name"), "%" + criteria.getLike() + "%"), builder.like(root.get("email"), "%" + criteria.getLike() + "%")));
        if (criteria.getEmail() != null)
            users.where((builder, root) -> builder.equal(root.get("email"), criteria.getEmail()));
        return users;
    }

    @Override
    public UserInfo createOrUpdate(UserInfo item) {
        return this.getDatabaseService().createOrUpdate(item, UserInfo.class);
    }

    @Override
    public UserInfo find(UUID id) {
        return this.getDatabaseService().getQueryable(UserInfo.class).where((builder, root) -> builder.equal(root.get("id"), id)).getSingle();
    }

    @Override
    public void delete(UserInfo item) {
        this.getDatabaseService().delete(item);
    }

    @Override
    public QueryableList<UserInfo> asQueryable() {
        return this.getDatabaseService().getQueryable(UserInfo.class);
    }
}