package eu.eudat.dao.entities.security;

import eu.eudat.dao.DatabaseAccess;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.entities.Credential;
import eu.eudat.entities.UserInfo;
import eu.eudat.models.login.Credentials;
import eu.eudat.queryable.QueryableList;
import eu.eudat.security.validators.TokenValidatorFactoryImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.UUID;


@Component("credentialDao")
public class CredentialDaoImpl extends DatabaseAccess<Credential> implements CredentialDao {

    @Autowired
    public CredentialDaoImpl(DatabaseService<Credential> databaseService) {
        this.setDatabaseService(databaseService);
    }

    @Override
    public Credential createOrUpdate(Credential item) {
        return this.getDatabaseService().createOrUpdate(item, Credential.class);
    }

    @Override
    public Credential find(UUID id) {
        return this.getDatabaseService().getQueryable(Credential.class).where((builder, root) -> builder.equal(root.get("id"), id)).getSingleOrDefault();
    }

    @Override
    public Credential getLoggedInCredentials(Credentials credentials) {
        return this.getDatabaseService().getQueryable(Credential.class).where(((builder, root) ->
                builder.and(
                        builder.equal(root.get("publicValue"), credentials.getUsername()),
                        builder.equal(root.get("secret"), credentials.getSecret()),
                        builder.equal(root.get("provider"), TokenValidatorFactoryImpl.LoginProvider.NATIVELOGIN.getValue())
                ))).getSingleOrDefault();
    }

    @Override
    public void delete(Credential item) {
        this.getDatabaseService().delete(item);
    }

    @Override
    public QueryableList<Credential> asQueryable() {
        return this.getDatabaseService().getQueryable(Credential.class);
    }
}
