package eu.eudat.managers;

import eu.eudat.dao.entities.UserInfoDao;
import eu.eudat.entities.UserRole;
import eu.eudat.exceptions.security.UnauthorisedException;
import eu.eudat.models.helpers.common.DataTableData;
import eu.eudat.models.login.Credentials;
import eu.eudat.models.security.Principal;
import eu.eudat.models.userinfo.UserInfoTableRequestItem;
import eu.eudat.models.userinfo.UserListingModel;
import eu.eudat.queryable.QueryableList;
import eu.eudat.services.ApiContext;
import eu.eudat.services.AuthenticationService;
import eu.eudat.utilities.builders.DomainModelConverter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import eu.eudat.utilities.builders.XmlBuilder;

import java.util.List;

public class UserManager {
    public static eu.eudat.models.user.composite.DatasetProfile generateDatasetProfileModel(eu.eudat.entities.DatasetProfile profile) {
        Document viewStyleDoc = XmlBuilder.fromXml(profile.getDefinition());
        Element root = (Element) viewStyleDoc.getDocumentElement();
        eu.eudat.entities.xmlmodels.datasetprofiledefinition.ViewStyleModel viewstyle = new eu.eudat.entities.xmlmodels.datasetprofiledefinition.ViewStyleModel().fromXml(root);

        eu.eudat.models.user.composite.DatasetProfile datasetprofile = new eu.eudat.models.user.composite.DatasetProfile();
        datasetprofile.buildProfile(viewstyle);

        return datasetprofile;
    }

    public static DataTableData<UserListingModel> getPaged(UserInfoDao userInfoDao, UserInfoTableRequestItem userInfoTableRequestItem) throws Exception {
        QueryableList<eu.eudat.entities.UserInfo> users = userInfoDao.getWithCriteria(userInfoTableRequestItem.getCriteria());
        QueryableList<eu.eudat.entities.UserInfo> pagedUsers = PaginationManager.applyPaging(users, userInfoTableRequestItem);

        List<UserListingModel> modelUsers = new DomainModelConverter<eu.eudat.entities.UserInfo, UserListingModel>().fromDataModel(pagedUsers.toList(), UserListingModel.class);
        DataTableData<UserListingModel> dataTableData = new DataTableData<>();
        dataTableData.setData(modelUsers);
        dataTableData.setTotalCount(users.count());
        return dataTableData;
    }

    public static void editRoles(ApiContext apiContext, UserListingModel user) {
        eu.eudat.entities.UserInfo userInfo = apiContext.getDatabaseRepository().getUserInfoDao().find(user.getId());
        userInfo.getUserRoles().stream().forEach(item -> apiContext.getDatabaseRepository().getUserRoleDao().delete(item));
        for (Integer role : user.getAppRoles()) {
            UserRole userRole = new UserRole();
            userRole.setRole(role);
            userRole.setUserInfo(userInfo);
            apiContext.getDatabaseRepository().getUserRoleDao().createOrUpdate(userRole);
        }
    }

    public static Principal authenticate(AuthenticationService authenticationService, Credentials credentials) {
        Principal principal = authenticationService.Touch(credentials);
        if (principal == null) throw new UnauthorisedException("Could not Sign In User");
        return principal;
    }
}
